/**********************************************************************
**   DEMO.C                                                          **
**   Example program using Interupt Driven Serial Port Routines      **
**   Written 2/24/94 by Milton Watts                                 **
**   QUARZDYNE, Inc.                                                 **
**   Modified 4/26/96: Add variable baud rate option                 **
**            4/15/96: Increase timeout time for longer gatetimes    **
**********************************************************************/


#include <time.h>   /* clock(), CLK_TIC */
#include <bios.h>   /* _bios_equiplist */
#include <string.h> /* strtok() */
#include <stdlib.h> /* atof() */
#include <stddef.h> /* NULL */

#include "serial.h"

int wait_for_message(int seconds);
int parce(char *buf, char *ptr[], int nPtrs);

#define MAXDEVICE 5
#define LINES 5

int main(int argc, char * argv[]) {
	int port,nports, rs485=0;
	double gatetime,maxgate=1.0;
	int timeout=2;
	char *fmt = "%18s%18s%18s%18s";
	char *ptr[6],buf[1024];
	char *address[MAXDEVICE]={"01","02","03","04","05"};
	char nDevices=0;
	int i,j,x,y;
	char usage[]=	"\nUsage: DEMO COMn [#nn] [#nn] [#nn] [#nn] [#nn] [bnnnn] [e]\n"
						"     COMn indicates the serial port (n=1 or 2)\n"
						"     [#nn] is the logical device interface address (Up to 5)\n"
						"     [bnnnn] allows for baud rate setting nnnn=300|600|1200|2400|4800|9600\n"
						"     [e] allows for RS485 interfaces which echo outgoing characters\n"
						"     brackets indicate optional parameters\n"
						"     defaults: #01, 1200 Baud\n"
	;
	unsigned baud = BAUD_1200;  // default baud rate

	if (argc < 2 || strnicmp (argv[1],"COM",3) != 0) {
		printf(usage);
		return 1;
	}
	port = argv[1][3]-'1';
	nports = ( _bios_equiplist() & 0x0E00 ) >> 9;
	if (port < 0 || port >=nports ) {
		printf("\nCommunications port %s not found\n",argv[1]);
		return 1;
	}

	for (i=2;i<argc;++i) {        /* read optional parameters */
		if ( strnicmp(argv[i],"E",1) == 0 ) rs485 = 1;
		else if ( *argv[i] == '#' && strlen(argv[i]) == 3 && nDevices<MAXDEVICE)
				strcpy(address[nDevices++], argv[i]+1);
		else if ( *argv[i] == 'b' || *argv[i] == 'B') {
			switch(*(argv[i]+1) ) {
			case '1':
				baud=_COM_1200;
				break;
			case '2':
				baud=_COM_2400;
				break;
			case '3':
				baud=_COM_300;
				break;
			case '4':
				baud=_COM_4800;
				break;
			case '6':
				baud=_COM_600;
				break;
			case '9':
				baud=_COM_9600;
				break;
			default:
				printf("\nInvalid baud rate specifier %s\n",argv[i]);
				printf(usage);
				exit(1);
			}
		} else {
			printf(usage);
			return 1;
		};
	};
	if (nDevices==0) nDevices=1;
	/* first make sure the communications link is open */
	baud |= COMM_N81;  // settings = baud + No Parity + 8 Data bits + 1 stop
	if ( comm_init(port, baud, rs485)==0 ) {
		printf("\nCommunications Port Cannot be initialized\n");
		printf("%s\n",error_message( comm_error(0) ) );
		return 1;
	}

	clrscr();

	for (i=0;i<nDevices;++i){
		send_message(address[i],"ver");  /* check communications */
		if ( wait_for_message(timeout) == 0 ) {
			printf("\n\n\nNo response on COM%d address #%s\n", port+1, address[i]);
			if (i>0) printf("Check device address\n");
			else printf("Check cables, device address and COMn setting\n");
			comm_restore();
			return 1;
		}
		rcv_message(buf);
		if (*buf!= 'V') {
			printf("\n\n\nInvalid response on COM%d address #%s\n%s\n", port+1, address[i], buf);
			comm_restore();
			return 1;
		}

		gotoxy(1,1+LINES*i);
		buf[8]='\0';   /* Print only the version number */
		printf(" #%s  QUARTZDYNE Inc. Transducer Interface %s Ŀ", address[i],buf);
	for (j=0;j<3;++j)
		printf("                                                                              ");
		printf("");
		x=2;
		y=wherey()-4;

		if ( buf[4] >= '2') {  /* ver>=2.01, check gatetime */
			send_message(address[i],"ga");     /* check gatetime */
			if (wait_for_message(timeout) !=0) {
				rcv_message(buf);
				if (buf[0] !='E') {
					gatetime = atof(buf);
					if (gatetime>maxgate) maxgate=gatetime;
				}
			}
		}
		send_message(address[i],"ct1;ct2");     /* read column names */
		if (wait_for_message(timeout) !=0) {
			rcv_message(buf);
			parce(buf,ptr,2);   /* break response into individuals strings */
			gotoxy(x,y++);
			cprintf(fmt,ptr[0],ptr[1],"Frequency 1","Frequency 2");
		} else y++;

		send_message(address[i],"un1;un2");   /* read unit names */
		if (wait_for_message(timeout) !=0) {
			rcv_message(buf);
			parce(buf,ptr,2);
			gotoxy(x,y++);
			cprintf(fmt,ptr[0],ptr[1],"Hz","Hz");
		} else y++;

	}

	timeout=(int) ( maxgate + 1.5 );  // set an appropriate timout delay
	if (timeout<2) timeout=2;

	i=0;
	while (!kbhit() ) {
		y=4+LINES*i;
		/* make sure no stray messages are pending */
		while (message_rcvd(0)) rcv_message(buf);

		send_message(address[i],"d1;d2;d3;d4");
		if ( wait_for_message(timeout) ) {
			rcv_message(buf); /* read data */
			parce(buf,ptr,4);
			gotoxy(x,y);
			cprintf(fmt,ptr[0],ptr[1],ptr[2],ptr[3]);

		} else {
			gotoxy(x,y);
			cprintf(fmt,"No Reply","","","");
		}
		if (++i >= nDevices) i=0;
	}
	if ( ( i=getch() ) == 0 ) i=getch() + 0x100;
	comm_restore();
	gotoxy(1,1+LINES*nDevices);

	return(0);
}


int wait_for_message(int seconds) {
	/* monitors message_rcvd() until either a message
		has been detected or until the number of seconds has elapsed.
		returns 1 if a message is detected, 0 otherwise.
	*/
	clock_t timeout;

	timeout= clock() + ( clock_t ) ( seconds * CLK_TCK ) ;
		 /* timeout in clock tics since program start */
	while ( message_rcvd(0) <=0 ) {  /* check for message first */
		if ( clock() > timeout ) return 0;
	}
	return 1;
}

int parce(char *buf, char *ptr[], int nPtrs) {
	/* Replaces commas and LF characters in buf with NULL characters and
		 sets the pointers in *ptr[] to the first letter of each substring.
		 No more than nPtrs will be set.  If more substrings exist, they
		 will be counted, but pointers will not be set.  If fewer than nPtrs
		 substrings exist, the remaining pointers will be set to the empty
		 string.  The return value is the number of substrings found regardless
		 of how many pointers were initialized.  Adjacent commas will create
		 countable, zero-lengthed substrings.
	*/

	int i=0;
	char c;
	ptr[i++]=buf;                           /* first pointer to beginning */
	while ( ( c = *buf++ ) != '\0' ) {          /* traverse entire string */
		if ( c == ',' || c == '\n') {             /* look for  ',' and '\n' */
			*(buf-1)='\0';                          /* replace with with '\0' */
			if ( i < nPtrs ) ptr[i]=buf;            /* store next pointer */
			if ( c != '\n' || *buf != '\0' ) ++i;   /* count number found */
		}
	}
	--buf;                                /* point to terminating character */
	while (nPtrs>i) ptr[--nPtrs]=buf;     /* set remaining pointers to ""  */
	return i;
}