/**********************************************************************
**   SERIAL.H                                                        **
**   Interupt Driven Serial Port Routines Header File                **
**   Written 2/24/94 by Milton Watts                                 **
**   QUARZDYNE, Inc.                                                 **
**                                                                   **
**********************************************************************/

#ifndef _SERIAL_
#define _SERIAL_

/* DOS BIOS communications parameters bit fields */

#define BITS_7      0x02 /* number of data bits*/
#define BITS_8      0x03
#define STOP_BIT_1  0x00 /* stop bits */
#define STOP_BIT_2  0x04
#define NO_PARITY   0x00 /* parity */
#define ODD_PARITY  0x08
#define EVEN_PARITY 0x18

#define BAUD_110    0x00
#define BAUD_150    0x20
#define BAUD_300    0x40
#define BAUD_600    0x60
#define BAUD_1200   0x80
#define BAUD_2400   0xA0
#define BAUD_4800   0xC0
#define BAUD_9600   0xE0

#define QDYNE_SETTINGS (BAUD_1200 | NO_PARITY | BITS_8 | STOP_BIT_1)
#define COMM_N81 (NO_PARITY | BITS_8 | STOP_BIT_1)

#define COM1 0
#define COM2 1

#define RBUF_SIZE 512     /* just big enough for 1 coefficeint file */
#define TBUF_SIZE 512

/* error codes returned by comm_error() */

#define LSR_BRK   0x10   /* break detected */
#define LSR_FE    0x80   /* framing error */
#define LSR_PE    0x04   /* parity error */
#define LSR_OE    0x02   /* overrunn error */

#define RBUF_OVER        0x0100  /* receive buffer overrunn error */
#define INVALID_PORT_ID  0x0200  /* invalid port id (not 0 or 1) */
#define PORT_NOT_FOUND   0x0400  /* BIOS does not know port address */

int comm_init(unsigned port, unsigned settings, unsigned RS485);
void comm_restore(void);
unsigned comm_error(unsigned flag);  /* flag should always be zero */
char* error_message(unsigned flag);

void send_message(char *address, char *message);
int rcv_message(char *message);
int send_char(char c);
int rcv_char(char *c);

int message_sent(void);
int message_rcvd(int count);  /* 'count' should always be zero */

#endif _SERIAL_